@@ -15,12 +15,14 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import cairo
-from gi.repository import Gtk, Gdk, Gio, GdkPixbuf, Pango
+import cairo, random, math
+from gi.repository import Gtk, Gdk, Gio, GdkPixbuf, Pango, GLib
+from .gi_composites import GtkTemplate
 from .history_manager import DrHistoryManager
 from .selection_manager import DrSelectionManager
 from .properties import DrPropertiesDialog
 from .utilities import InvalidFileFormatException
+from .utilities_overlay import utilities_generic_canvas_outline
 
 class DrMotionBehavior():
 	_LIMIT = 10
@@ -37,23 +39,27 @@ class NoPixbufNoChangeException(Exception):
 
 ################################################################################
 
-@Gtk.Template(resource_path='/com/github/maoschanz/drawing/ui/image.ui')
+@GtkTemplate(ui='/com/github/maoschanz/drawing/ui/image.ui')
 class DrImage(Gtk.Box):
 	__gtype_name__ = 'DrImage'
 
-	_drawing_area = Gtk.Template.Child()
-	_h_scrollbar = Gtk.Template.Child()
-	_v_scrollbar = Gtk.Template.Child()
+	_drawing_area = GtkTemplate.Child()
+	_h_scrollbar = GtkTemplate.Child()
+	_v_scrollbar = GtkTemplate.Child()
 
 	SCALE_FACTOR = 1.0 # XXX doesn't work well enough to be anything else
 
 	def __init__(self, window, **kwargs):
 		super().__init__(**kwargs)
+		self.init_template()
 		self.window = window
 
 		self.gfile = None
 		self.filename = None
 
+		self._framerate_hint = 1.0
+		self._fps_counter = 0
+
 		self._init_drawing_area()
 
 		self._update_background_color()
@@ -96,8 +102,9 @@ class DrImage(Gtk.Box):
 		rgba = self.window.gsettings.get_strv('ui-background-rgba')
 		self._bg_rgba = (float(rgba[0]), float(rgba[1]), \
 		                                         float(rgba[2]), float(rgba[3]))
-		# We remember this data here for performance: it's used by the `on_draw`
-		# method which is called a lot, and reading a gsettings costs a lot.
+		# We remember this data here for performance: it will eb used by the
+		# `on_draw` method which is called a lot, and reading a gsettings costs
+		# a lot.
 
 	def _update_zoom_behavior(self, *args):
 		self._ctrl_to_zoom = self.window.gsettings.get_boolean('ctrl-zoom')
@@ -131,35 +138,35 @@ class DrImage(Gtk.Box):
 	def init_background(self, width, height, background_rgba):
 		self.init_image_common()
 		self._history.set_initial_operation(background_rgba, None, width, height)
-		self.restore_first_pixbuf()
+		self.restore_last_state()
 
 	def try_load_pixbuf(self, pixbuf):
 		self.init_image_common()
 		self._load_pixbuf_common(pixbuf)
-		self.restore_first_pixbuf()
+		self.restore_last_state()
 		self.update_title()
 
 	def _new_blank_pixbuf(self, w, h):
 		return GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, w, h)
 
-	def restore_first_pixbuf(self):
+	def restore_last_state(self):
 		"""Set the last saved pixbuf from the history as the main_pixbuf. This
 		is used to rebuild the picture from its history."""
-		last_saved_pixbuf_op = self._history.get_last_saved_state()
+		state_op = self._history.get_last_saved_state()
 
 		# restore the state found in the history
-		pixbuf = last_saved_pixbuf_op['pixbuf']
-		width = last_saved_pixbuf_op['width']
-		height = last_saved_pixbuf_op['height']
+		pixbuf = state_op['pixbuf']
+		width = state_op['width']
+		height = state_op['height']
 		self.set_temp_pixbuf(self._new_blank_pixbuf(1, 1))
 		self.selection.init_pixbuf()
 		self.surface = cairo.ImageSurface(cairo.Format.ARGB32, width, height)
 		if pixbuf is None:
 			# no pixbuf in the operation: the restored state is a blank one
-			r = last_saved_pixbuf_op['red']
-			g = last_saved_pixbuf_op['green']
-			b = last_saved_pixbuf_op['blue']
-			a = last_saved_pixbuf_op['alpha']
+			r = state_op['red']
+			g = state_op['green']
+			b = state_op['blue']
+			a = state_op['alpha']
 			self.set_main_pixbuf(self._new_blank_pixbuf(width, height))
 			cairo_context = cairo.Context(self.surface)
 			cairo_context.set_source_rgba(r, g, b, a)
@@ -167,10 +174,11 @@ class DrImage(Gtk.Box):
 			self.update()
 			self.set_surface_as_stable_pixbuf()
 		else:
-			self.set_main_pixbuf(last_saved_pixbuf_op['pixbuf'].copy())
+			self.set_main_pixbuf(state_op['pixbuf'].copy())
 			self.use_stable_pixbuf()
 
 	############################################################################
+	# (re)loading the pixbuf of a given file ###################################
 
 	def _load_pixbuf_common(self, pixbuf):
 		if not pixbuf.get_has_alpha():
@@ -211,6 +219,10 @@ class DrImage(Gtk.Box):
 			# ya pas de fenêtre) ouvrir un truc respectant les settings, plutôt
 			# qu'un petit pixbuf corrompu
 		self.try_load_pixbuf(pixbuf)
+		self._can_reload()
+
+	def _can_reload(self):
+		self.set_action_sensitivity('reload_file', self.gfile is not None)
 
 	############################################################################
 	# Image title and tab management ###########################################
@@ -284,6 +296,10 @@ class DrImage(Gtk.Box):
 	def show_properties(self):
 		DrPropertiesDialog(self.window, self)
 
+	def update_image_wide_actions(self):
+		self.update_history_sensitivity()
+		self._can_reload()
+
 	############################################################################
 	# History management #######################################################
 
@@ -348,6 +364,9 @@ class DrImage(Gtk.Box):
 
 	def on_draw(self, area, cairo_context):
 		"""Signal callback. Executed when self._drawing_area is redrawn."""
+		if self.window.devel_mode:
+			self._fps_counter += 1
+
 		# Background color
 		cairo_context.set_source_rgba(*self._bg_rgba)
 		cairo_context.paint()
@@ -355,16 +374,18 @@ class DrImage(Gtk.Box):
 		# Zoom level
 		cairo_context.scale(self.zoom_level, self.zoom_level)
 
-		# TODO transform tools may appreciate to draw *before* the image (issue
-		# when the canvas becomes larger)
-
 		# Image (with scroll position)
 		cairo_context.set_source_surface(self.get_surface(), \
 		                                 -1 * self.scroll_x, -1 * self.scroll_y)
 		cairo_context.paint()
 
-		# What the tool is painting
-		self.active_tool().on_draw(area, cairo_context)
+		# What the tool shows on the canvas, upon what it paints, for example an
+		# overlay to imply how to interact with a previewed operation.
+		self.active_tool().on_draw_above(area, cairo_context)
+
+		# Limit of the canvas (for readability)
+		utilities_generic_canvas_outline(cairo_context, self.get_pixbuf_width(), \
+		                              self.get_pixbuf_height(), self.zoom_level)
 
 	def on_press_on_area(self, area, event):
 		"""Signal callback. Executed when a mouse button is pressed on
@@ -402,7 +423,7 @@ class DrImage(Gtk.Box):
 			# implicitely impossible if not self._is_pressed
 			event_x, event_y = self.get_event_coords(event)
 			self.active_tool().on_motion_on_area(event, self.surface, event_x, event_y)
-			self.update() # <<< comment this for better perfs
+			self.update()
 
 		else: # self.motion_behavior == DrMotionBehavior.SLIP:
 			self.scroll_x = self._slip_init_x
@@ -435,9 +456,6 @@ class DrImage(Gtk.Box):
 
 	def update(self):
 		# print('image.py: _drawing_area.queue_draw')
-		# TODO immensément utilisée, mais qui ne scale pas : ça gêne clairement
-		# l'utilisation pour les trop grandes images, il faudrait n'update que
-		# la partie qui change, ou au pire que la partie affichée
 		self._drawing_area.queue_draw()
 
 	def get_surface(self):
@@ -454,16 +472,18 @@ class DrImage(Gtk.Box):
 		self.update()
 
 	def set_surface_as_stable_pixbuf(self):
-		# print('image.py: set_surface_as_stable_pixbuf')
-		self.main_pixbuf = Gdk.pixbuf_get_from_surface(self.surface, 0, 0, \
-		                    self.surface.get_width(), self.surface.get_height())
+		w = self.surface.get_width()
+		h = self.surface.get_height()
+		self.main_pixbuf = Gdk.pixbuf_get_from_surface(self.surface, 0, 0, w, h)
+		# self._framerate_hint = math.sqrt(500000 / (w * h))
+		# print("hint :", self._framerate_hint)
 
 	def use_stable_pixbuf(self):
-		# print('image.py: use_stable_pixbuf')
-		# TODO immensément utilisée, mais qui ne scale pas : ça gêne clairement
-		# l'utilisation pour les trop grandes images, il faudrait n'update que
-		# la partie qui change, ou au pire que la partie affichée
+		"""This is called by tools' `restore_pixbuf`, so at the beginning of
+		each operation (even unapplied)."""
+		# maybe the "scale" parameter should be 1 instead of 0
 		self.surface = Gdk.cairo_surface_create_from_pixbuf(self.main_pixbuf, 0, None)
+		# print('image.py: use_stable_pixbuf')
 		self.surface.set_device_scale(self.SCALE_FACTOR, self.SCALE_FACTOR)
 
 	def get_pixbuf_width(self):
