@@ -18,6 +18,7 @@
 # Import libs
 import os, traceback
 from gi.repository import Gtk, Gdk, Gio, GdkPixbuf, GLib
+from .gi_composites import GtkTemplate
 
 # Import tools
 from .tool_arc import ToolArc
@@ -37,7 +38,6 @@ from .tool_crop import ToolCrop
 from .tool_filters import ToolFilters
 from .tool_rotate import ToolRotate
 from .tool_scale import ToolScale
-# from .tool_skew import ToolSkew
 
 from .select_rect import ToolRectSelect
 from .select_free import ToolFreeSelect
@@ -76,27 +76,28 @@ DEFAULT_TOOL_ID = 'pencil'
 
 ################################################################################
 
-@Gtk.Template(resource_path=UI_PATH+'window.ui')
+@GtkTemplate(ui=UI_PATH+'window.ui')
 class DrWindow(Gtk.ApplicationWindow):
 	__gtype_name__ = 'DrWindow'
 
 	gsettings = Gio.Settings.new('com.github.maoschanz.drawing')
 
 	# Window empty widgets
-	tools_flowbox = Gtk.Template.Child()
-	toolbar_box = Gtk.Template.Child()
-	info_bar = Gtk.Template.Child()
-	info_label = Gtk.Template.Child()
-	info_action = Gtk.Template.Child()
-	notebook = Gtk.Template.Child()
-	bottom_panes_box = Gtk.Template.Child()
-	unfullscreen_btn = Gtk.Template.Child()
-	bottom_meta_box = Gtk.Template.Child()
-	tools_scrollable_box = Gtk.Template.Child()
-	tools_nonscrollable_box = Gtk.Template.Child()
+	tools_flowbox = GtkTemplate.Child()
+	toolbar_box = GtkTemplate.Child()
+	info_bar = GtkTemplate.Child()
+	info_label = GtkTemplate.Child()
+	info_action = GtkTemplate.Child()
+	notebook = GtkTemplate.Child()
+	bottom_panes_box = GtkTemplate.Child()
+	unfullscreen_btn = GtkTemplate.Child()
+	bottom_meta_box = GtkTemplate.Child()
+	tools_scrollable_box = GtkTemplate.Child()
+	tools_nonscrollable_box = GtkTemplate.Child()
 
 	def __init__(self, **kwargs):
 		super().__init__(**kwargs)
+		self.init_template()
 		self.app = kwargs['application']
 
 		self.pointer_to_current_page = None # this ridiculous hack allows to
@@ -104,6 +105,8 @@ class DrWindow(Gtk.ApplicationWindow):
 		                                      # notebook widget being pure shit
 		self.active_tool_id = None
 		self._is_tools_initialisation_finished = False
+		self.devel_mode = False
+		self.should_track_framerate = False
 
 		if self.gsettings.get_boolean('maximized'):
 			self.maximize()
@@ -135,6 +138,7 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.saving_manager = DrSavingManager(self)
 		self.printing_manager = DrPrintingManager(self)
 
+		self.devel_mode = self.gsettings.get_boolean('devel-only')
 		self.add_all_win_actions()
 		self._init_tools()
 		self.connect_signals()
@@ -145,7 +149,7 @@ class DrWindow(Gtk.ApplicationWindow):
 		# can continue normally.
 		try:
 			if get_cb:
-				self.build_image_from_clipboard()
+				self.delayed_build_from_clipboard()
 			elif gfile is not None:
 				self.build_new_tab(gfile=gfile)
 			else:
@@ -157,7 +161,7 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.set_picture_title()
 		self._try_show_release_notes()
 
-		# has to return False to be removed from the mainloop immediatly
+		# has to return False to be removed from the mainloop immediately
 		return False
 
 	def _try_show_release_notes(self):
@@ -172,8 +176,8 @@ class DrWindow(Gtk.ApplicationWindow):
 		                                   "would you like to read what's new?")
 		dialog.add_string(label % current_version)
 
-		no_id = dialog.set_action(_("No"), None, False)
-		later_id = dialog.set_action(_("Later"), None, False)
+		no_id = dialog.set_action(_("No"), None)
+		later_id = dialog.set_action(_("Later"), None)
 		yes_id = dialog.set_action(_("Yes"), 'suggested-action', True)
 		result = dialog.run()
 		dialog.destroy()
@@ -211,7 +215,6 @@ class DrWindow(Gtk.ApplicationWindow):
 		self._load_tool('crop', ToolCrop, disabled_tools, dev)
 		self._load_tool('scale', ToolScale, disabled_tools, dev)
 		self._load_tool('rotate', ToolRotate, disabled_tools, dev)
-		# self._load_tool('skew', ToolSkew, disabled_tools, dev)
 		self._load_tool('filters', ToolFilters, disabled_tools, dev)
 
 		# Side pane buttons for tools, and their menubar items if they don't
@@ -309,6 +312,16 @@ class DrWindow(Gtk.ApplicationWindow):
 			self.set_picture_title()
 		dialog.destroy()
 
+	def delayed_build_from_clipboard(self, *args):
+		"""Calls `async_build_from_clipboard` asynchronously."""
+		self.build_new_tab() # temporary image to avoid errors when the window
+		# finishes its initialisation.
+		GLib.timeout_add(500, self.async_build_from_clipboard, {})
+
+	def async_build_from_clipboard(self, content_params):
+		self.get_active_image().try_close_tab()
+		self.build_image_from_clipboard()
+
 	def build_image_from_clipboard(self, *args):
 		"""Open a new tab with the image in the clipboard. If the clipboard is
 		empty, the new image will be blank."""
@@ -326,7 +339,7 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.build_new_tab(pixbuf=pixbuf)
 
 	def build_new_tab(self, gfile=None, pixbuf=None, \
-		           width=200, height=200, background_rgba=[1.0, 0.0, 0.0, 1.0]):
+		           width=200, height=200, background_rgba=[0.5, 0.5, 0.5, 0.5]):
 		"""Open a new tab with an optional file to open in it."""
 		new_image = DrImage(self)
 		self.notebook.append_page(new_image, new_image.build_tab_widget())
@@ -335,9 +348,9 @@ class DrWindow(Gtk.ApplicationWindow):
 			new_image.try_load_file(gfile)
 		elif pixbuf is not None:
 			new_image.try_load_pixbuf(pixbuf)
-			# XXX dans l'idéal on devrait ne rien ouvrir non ? ou si besoin (si
-			# ya pas de fenêtre) ouvrir un truc respectant les settings, plutôt
-			# qu'un petit pixbuf rouge
+			# TODO dans l'idéal si on passe par les actions du clipboard, on
+			# devrait n'ouvrir que si il n'y a pas de fenêtre, et ouvrir un truc
+			# respectant les settings, plutôt qu'un petit pixbuf gris
 		else:
 			new_image.init_background(width, height, background_rgba)
 		self._update_tabs_visibility()
@@ -350,7 +363,7 @@ class DrWindow(Gtk.ApplicationWindow):
 		# print("changement d'image")
 		self.set_picture_title(args[1].update_title())
 		self.minimap.set_zoom_label(args[1].zoom_level * 100)
-		args[1].update_history_sensitivity()
+		args[1].update_image_wide_actions()
 		# On devrait être moins bourrin et conserver la sélection # TODO ?
 
 	def update_tabs_menu_section(self, *args):
@@ -417,8 +430,8 @@ class DrWindow(Gtk.ApplicationWindow):
 
 	def connect_signals(self):
 		# Closing the info bar
-		self.info_bar.connect('close', self.hide_message)
-		self.info_bar.connect('response', self.hide_message)
+		self.info_bar.connect('close', self._hide_message)
+		self.info_bar.connect('response', self._hide_message)
 
 		# Closing the window
 		self.connect('delete-event', self.on_close)
@@ -430,8 +443,8 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.gsettings.connect('changed::show-labels', self.on_show_labels_setting_changed)
 		self.gsettings.connect('changed::deco-type', self.on_layout_changed)
 		self.gsettings.connect('changed::big-icons', self.on_icon_size_changed)
-		# self.gsettings.connect('changed::preview-size', self.show_info_settings)
-		# self.gsettings.connect('changed::devel-only', self.show_info_settings)
+		self.gsettings.connect('changed::preview-size', self.show_info_settings)
+		# devel-only' isn't connected because it's auto-updated to False
 		self.gsettings.connect('changed::disabled-tools', self.show_info_settings)
 		self.gsettings.connect('changed::dark-theme-variant', self._update_theme_variant)
 		# Other settings are connected in DrImage
@@ -462,15 +475,14 @@ class DrWindow(Gtk.ApplicationWindow):
 			current_item = current_item.get_item_link(item[1], link_type)
 		return current_item
 
-	def add_action_simple(self, action_name, callback, shortcuts):
+	def add_action_simple(self, action_name, callback, shortcuts=[]):
 		"""Convenient wrapper method adding a stateless action to the window. It
 		will be named 'action_name' (string) and activating the action will
 		trigger the method 'callback'."""
 		action = Gio.SimpleAction.new(action_name, None)
 		action.connect('activate', callback)
 		self.add_action(action)
-		if shortcuts is not None:
-			self.app.set_accels_for_action('win.' + action_name, shortcuts)
+		self.app.set_accels_for_action('win.' + action_name, shortcuts)
 
 	def add_action_boolean(self, action_name, default, callback):
 		"""Convenient wrapper method adding a stateful action to the window. It
@@ -513,8 +525,8 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.app.set_accels_for_action('win.fullscreen', ['F11'])
 
 		self.add_action_simple('reload_file', self.action_reload, ['<Ctrl>r'])
-		self.add_action_simple('properties', self.action_properties, None)
-		self.add_action_simple('unfullscreen', self.action_unfullscreen, None)
+		self.add_action_simple('properties', self.action_properties)
+		self.add_action_simple('unfullscreen', self.action_unfullscreen)
 
 		self.add_action_simple('go_up', self.action_go_up, ['<Ctrl>Up'])
 		self.add_action_simple('go_down', self.action_go_down, ['<Ctrl>Down'])
@@ -527,7 +539,7 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.add_action_simple('zoom_opti', self.action_zoom_opti, ['<Ctrl>0', '<Ctrl>KP_0'])
 
 		self.add_action_simple('new_tab', self.build_new_image, ['<Ctrl>t'])
-		self.add_action_simple('new_tab_custom', self.build_new_custom, None)
+		self.add_action_simple('new_tab_custom', self.build_new_custom)
 		self.add_action_simple('new_tab_selection', \
 		                    self.build_image_from_selection, ['<Ctrl><Shift>t'])
 		self.add_action_simple('new_tab_clipboard', \
@@ -536,35 +548,35 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.add_action_simple('tab_left', self.action_tab_left, ['<Ctrl><Shift>Left'])
 		self.add_action_simple('tab_right', self.action_tab_right, ['<Ctrl><Shift>Right'])
 		self.add_action_simple('close_tab', self.action_close_tab, ['<Ctrl>w'])
-		self.add_action_simple('close', self.action_close_window, None)
+		self.add_action_simple('close', self.action_close_window)
 
 		self.add_action_simple('undo', self.action_undo, ['<Ctrl>z'])
 		self.add_action_simple('redo', self.action_redo, ['<Ctrl><Shift>z'])
 
 		self.add_action_simple('save', self.action_save, ['<Ctrl>s'])
-		self.add_action_simple('save_alphaless', self.action_save_alphaless, None)
+		self.add_action_simple('save_alphaless', self.action_save_alphaless)
 		self.add_action_simple('save_as', self.action_save_as, ['<Ctrl><Shift>s'])
-		self.add_action_simple('export_as', self.action_export_as, None)
+		self.add_action_simple('export_as', self.action_export_as)
 		self.add_action_simple('to_clipboard', self.action_export_cb, ['<Ctrl><Shift>c'])
-		self.add_action_simple('print', self.action_print, None)
+		self.add_action_simple('print', self.action_print)
 
 		self.add_action_simple('import', self.action_import, ['<Ctrl>i'])
 		self.add_action_simple('paste', self.action_paste, ['<Ctrl>v'])
 		self.add_action_simple('select_all', self.action_select_all, ['<Ctrl>a'])
 		self.add_action_simple('unselect', self.action_unselect, ['<Ctrl><Shift>a'])
-		#self.add_action_simple('selection_invert', self.action_selection_invert, None)
+		#self.add_action_simple('selection_invert', self.action_selection_invert)
 		self.add_action_simple('selection_cut', self.action_cut, ['<Ctrl>x'])
 		self.add_action_simple('selection_copy', self.action_copy, ['<Ctrl>c'])
 		self.add_action_simple('selection_delete', self.action_delete, ['Delete'])
 
-		self.add_action_simple('selection_export', self.action_selection_export, None)
+		self.add_action_simple('selection_export', self.action_selection_export)
 		self.add_action_simple('selection-replace-canvas', \
-		                             self.action_selection_replace_canvas, None)
+		                                   self.action_selection_replace_canvas)
 		self.add_action_simple('selection-expand-canvas', \
-		                              self.action_selection_expand_canvas, None)
+		                                    self.action_selection_expand_canvas)
 
 		self.add_action_simple('back_to_previous', self.back_to_previous, ['<Ctrl>b'])
-		self.add_action_simple('force_selection', self.force_selection, None)
+		self.add_action_simple('force_selection', self.force_selection)
 		self.add_action_simple('apply_transform', self.action_apply_transform, ['<Ctrl>Return'])
 
 		self.add_action_enum('active_tool', DEFAULT_TOOL_ID, self.on_change_active_tool)
@@ -579,9 +591,9 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.add_action_simple('size_more', self.action_size_more, ['<Ctrl><Shift>Up'])
 		self.add_action_simple('size_less', self.action_size_less, ['<Ctrl><Shift>Down'])
 
-		if self.gsettings.get_boolean('devel-only'):
-			self.add_action_simple('restore_pixbuf', self.action_restore, None)
-			self.add_action_simple('rebuild_from_histo', self.action_rebuild, None)
+		if self.devel_mode:
+			self.add_action_simple('restore_pixbuf', self.action_restore)
+			self.add_action_simple('rebuild_from_histo', self.action_rebuild)
 			self.add_action_simple('get_values', self.action_getvalues, ['<Ctrl>g'])
 
 		action = Gio.PropertyAction.new('active_tab', self.notebook, 'page')
@@ -689,9 +701,7 @@ class DrWindow(Gtk.ApplicationWindow):
 
 	def action_options_menu(self, *args):
 		"""This displays/hides the tool's options menu, and is implemented as an
-		action to ease the accelerator (shift+f10). This action could be
-		disable when the current pane doesn't contain the corresponding button,
-		but will not be."""
+		action to ease the accelerator (shift+f10)."""
 		self.options_manager.toggle_menu()
 
 	def _adapt_to_window_size(self, *args):
@@ -712,7 +722,7 @@ class DrWindow(Gtk.ApplicationWindow):
 		# Check whether or not the scrollbars should now be displayed
 		self.get_active_image().fake_scrollbar_update()
 
-	def hide_message(self, *args):
+	def _hide_message(self, *args):
 		self.prompt_message(False, '')
 
 	def prompt_message(self, show, label):
@@ -721,8 +731,8 @@ class DrWindow(Gtk.ApplicationWindow):
 		self.info_action.set_visible(False)
 		if show:
 			self.info_label.set_label(label)
-		if show and self.gsettings.get_boolean('devel-only') and label != "":
-			print('Drawing: ' + label)
+		if show and self.devel_mode and label != "":
+			print("Drawing: " + label)
 
 	def prompt_action(self, message, action_name='app.report_bug', action_label=_("Report a bug")):
 		"""Update the content of the info bar, including its actionable button
@@ -760,6 +770,7 @@ class DrWindow(Gtk.ApplicationWindow):
 			self.prompt_message(True, _("Middle-click, tap with 3 fingers, " + \
 			                           "or press F8 to show/hide controls.") + \
 			                           " " + _("Press F11 to exit fullscreen."))
+			# TODO à confirmer que 3 doigts c'est réel ^
 		else:
 			self.unfullscreen()
 		self._set_controls_hidden(shall_fullscreen)
@@ -895,12 +906,13 @@ class DrWindow(Gtk.ApplicationWindow):
 		if self.former_tool_id == self.active_tool_id:
 			self.force_selection()
 			# avoid cases where applying a transform tool keeps the tool active
-		self.tools[self.former_tool_id].row.set_active(True)
+		else:
+			self.tools[self.former_tool_id].row.set_active(True)
 
 	def _build_options_menu(self):
 		"""Build the active tool's option menus.
-		The first menu is the popover from the bottom bar. It can contain any
-		widget, or it can be build from a Gio.MenuModel
+		The first menu is the popover from the bottom bar. It can be built from
+		a Gio.MenuModel, or it can contain any widget.
 		The second menu is build from a Gio.MenuModel and is in the menubar (not
 		available with all layouts)."""
 		widget = self.active_tool().get_options_widget()
@@ -989,7 +1001,7 @@ class DrWindow(Gtk.ApplicationWindow):
 				self.try_load_file(gfile)
 			elif result == new_window_id:
 				self.app.open_window_with_content(gfile, False)
-		self.hide_message()
+		self._hide_message()
 
 	def file_chooser_open(self, *args):
 		"""Opens an "open" file chooser dialog, and return a GioFile or None."""
@@ -1009,8 +1021,8 @@ class DrWindow(Gtk.ApplicationWindow):
 		per image), or to import them (it will only import the first), or to
 		cancel (if the user dropped mistakenly)."""
 		dialog = DrMessageDialog(self)
-		cancel_id = dialog.set_action(_("Cancel"), None, False)
-		open_id = dialog.set_action(_("Open"), None, False)
+		cancel_id = dialog.set_action(_("Cancel"), None)
+		open_id = dialog.set_action(_("Open"), None)
 		import_id = dialog.set_action(_("Import"), None, True)
 
 		uris = data.get_uris()
